"""This module is the entry point for ostorlab CLI."""
import logging
from typing import Optional

import click

from ostorlab import configuration_manager

logger = logging.getLogger("CLI")


@click.group()
@click.pass_context
@click.version_option()
@click.option("--api-key", help="API key to login to the platform.", required=False)
@click.option("--proxy", "-X", help="Proxy to route HTTPS requests through.")
@click.option(
    "--tlsverify/--no-tlsverify",
    help="Control TLS server certificate verification.",
    default=True,
)
@click.option("-d", "--debug/--no-debug", help="Enable debug mode", default=False)
@click.option("-v", "--verbose/--no-verbose", help="Enable verbose mode", default=False)
def rootcli(
    ctx: click.core.Context,
    proxy: Optional[str] = None,
    tlsverify: Optional[bool] = True,
    debug: bool = False,
    verbose: bool = False,
    api_key: str = None,
) -> None:
    """Ostorlab is an open-source project to help automate security testing.\n
    Ostorlab standardizes interoperability between tools in a consistent, scalable, and performant way.
    """
    ctx.obj = {}
    ctx.obj["proxy"] = proxy
    ctx.obj["tlsverify"] = tlsverify
    ctx.obj["api_key"] = api_key
    # Configuration is a singleton class. One initiated with the provided API key, others will make use of it.
    conf_manager = configuration_manager.ConfigurationManager()
    conf_manager.api_key = api_key
    ctx.obj["config_manager"] = conf_manager

    if verbose is True:
        loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
        for l in loggers:
            l.setLevel(logging.INFO)
    if debug is True:
        loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
        for l in loggers:
            l.setLevel(logging.DEBUG)



"""Module for the command list inside the group scan.
This module takes care of listing all the remote or local scans.
Example of usage:
    - ostorlab scan list --source=source."""

import click
from ostorlab.cli.scan import scan
from ostorlab.cli import console as cli_console

console = cli_console.Console()


@scan.command()
@click.argument("scan_id", required=True)
@click.pass_context
def stop(ctx: click.core.Context, scan_id: int) -> None:
    """Stop a scan.\n
    Usage:\n
        - ostorlab scan --runtime=local stop --id=id
    """

    runtime_instance = ctx.obj["runtime"]
    with console.status("Stopping scan"):
        runtime_instance.stop(scan_id=scan_id)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/common/x509/x509.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3ostorlab/agent/message/proto/common/x509/x509.proto\x12(ostorlab.agent.message.proto.common.x509\"N\n\x07X509Org\x12\t\n\x01\x43\x18\x01 \x01(\t\x12\t\n\x01O\x18\x02 \x01(\t\x12\n\n\x02OU\x18\x03 \x01(\t\x12\n\n\x02\x43N\x18\x04 \x01(\t\x12\t\n\x01L\x18\x05 \x01(\t\x12\n\n\x02ST\x18\x06 \x01(\t\"1\n\x08Validity\x12\x12\n\nnot_before\x18\x01 \x01(\x04\x12\x11\n\tnot_after\x18\x02 \x01(\x04\"K\n\x0fX509Fingerprint\x12\x1d\n\x15\x66ingerprint_algorithm\x18\x01 \x01(\t\x12\x19\n\x11\x66ingerprint_value\x18\x02 \x01(\t\"=\n\rX509Extension\x12\x0b\n\x03oid\x18\x01 \x01(\t\x12\x10\n\x08\x63ritical\x18\x02 \x01(\x08\x12\r\n\x05value\x18\x03 \x01(\t\"G\n\rX509PublicKey\x12\x1c\n\x14public_key_algorithm\x18\x01 \x01(\t\x12\x18\n\x10public_key_value\x18\x02 \x01(\t\"E\n\rX509Signature\x12\x1b\n\x13signature_algorithm\x18\x01 \x01(\t\x12\x17\n\x0fsignature_value\x18\x02 \x01(\t\"\xc5\x04\n\x08X509Cert\x12\x16\n\x0eversion_number\x18\x01 \x01(\r\x12\x15\n\rserial_number\x18\x02 \x01(\t\x12\x41\n\x06issuer\x18\x03 \x01(\x0b\x32\x31.ostorlab.agent.message.proto.common.x509.X509Org\x12\x44\n\x08validity\x18\x04 \x01(\x0b\x32\x32.ostorlab.agent.message.proto.common.x509.Validity\x12\x42\n\x07subject\x18\x05 \x01(\x0b\x32\x31.ostorlab.agent.message.proto.common.x509.X509Org\x12K\n\npublic_key\x18\x06 \x01(\x0b\x32\x37.ostorlab.agent.message.proto.common.x509.X509PublicKey\x12J\n\tsignature\x18\x07 \x01(\x0b\x32\x37.ostorlab.agent.message.proto.common.x509.X509Signature\x12W\n\x14\x63omputed_fingerprint\x18\x08 \x03(\x0b\x32\x39.ostorlab.agent.message.proto.common.x509.X509Fingerprint\x12K\n\nextensions\x18\t \x03(\x0b\x32\x37.ostorlab.agent.message.proto.common.x509.X509Extension')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.common.x509.x509_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _X509ORG._serialized_start=97
  _X509ORG._serialized_end=175
  _VALIDITY._serialized_start=177
  _VALIDITY._serialized_end=226
  _X509FINGERPRINT._serialized_start=228
  _X509FINGERPRINT._serialized_end=303
  _X509EXTENSION._serialized_start=305
  _X509EXTENSION._serialized_end=366
  _X509PUBLICKEY._serialized_start=368
  _X509PUBLICKEY._serialized_end=439
  _X509SIGNATURE._serialized_start=441
  _X509SIGNATURE._serialized_end=510
  _X509CERT._serialized_start=513
  _X509CERT._serialized_end=1094
# @@protoc_insertion_point(module_scope)



"""Handles all authenticated API calls and behind the scenes operations such as authentication, validation, etc.

    Typical usage example:

    authenticated_runner = AuthenticatedAPIRunner(username, password, token_duration)
    authenticated_runner.authenticate()
"""
import datetime
import logging
from typing import Dict, Optional, Any

import click
import requests
import ubjson
import json

from ostorlab.apis import create_api_key
from ostorlab.apis import request as api_request
from ostorlab.apis.runners import login_runner
from ostorlab.apis.runners import runner
from ostorlab.cli import console as cli_console

AUTHENTICATED_GRAPHQL_ENDPOINT = "https://api.ostorlab.co/apis/graphql"

logger = logging.getLogger(__name__)
console = cli_console.Console()


class AuthenticationError(runner.Error):
    """Authentication Error."""


class AuthenticatedAPIRunner(runner.APIRunner):
    """Handles all authenticated API calls and behind the scenes operations such as authentication,
    validation, etc.
    """

    def __init__(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        token_duration: Optional[datetime.datetime] = None,
        proxy: Optional[str] = None,
        verify: bool = True,
        api_key: Optional[str] = None,
    ):
        """Constructs all the necessary attributes for the object.

        Args:
            username: the username (email) used to log in to use the token based authentication.
            password: the password used to log in to use the token based authentication.
            token_duration: The duration for which the token is valid
            (Can be in minutes, hours, days, or a combination of any two or all three).
            proxy: The proxy through which a request is made. Defaults to None.
            verify: Whether to verify the TLS certificate. Defaults to True.
            api_key: Use API KEY based authentication. Used if token is not defined.
        """
        super().__init__(proxy, verify)
        self._username = username
        self._password = password
        self._token_duration = token_duration
        self._api_key = api_key or self._configuration_manager.api_key
        self._token: Optional[str] = None
        self._otp_token: Optional[str] = None

    @property
    def endpoint(self) -> str:
        """API endpoint."""
        return AUTHENTICATED_GRAPHQL_ENDPOINT

    def authenticate(self) -> None:
        """Authenticates the user.

        Raises:
            AuthenticationError: If user credentials are not valid.
        """
        with console.status("Logging into your account"):
            login_api_runner = login_runner.LoginAPIRunner(
                username=self._username,
                password=self._password,
                otp_token=self._otp_token,
                proxy=self._proxy,
                verify=self._verify,
            )
            response = login_api_runner.login_user()

        if response.status_code != 200:
            field_errors = response.json().get("non_field_errors")
            if (
                field_errors is not None
                and field_errors[0] == 'Must include "otp_token"'
            ):
                self._otp_token = click.prompt(
                    "Please enter the OTP code from your authenticator app"
                )
                self.authenticate()
            else:
                logger.debug("response %s", response.content)
                raise AuthenticationError(response.status_code)
        else:
            self._token = response.json().get("token")
            with console.status("Generating API key"):
                api_key_response = self.execute(
                    create_api_key.CreateAPIKeyAPIRequest(self._token_duration)
                )
                console.success("API key generated")

            api_data = api_key_response["data"]["createApiKey"]["apiKey"]
            secret_key = api_data["secretKey"]
            api_key_id = api_data["apiKey"]["id"]
            expiry_date = api_data["apiKey"]["expiryDate"]

            self._api_key = secret_key
            with console.status("Persisting API key"):
                self._configuration_manager.set_api_data(
                    secret_key, api_key_id, expiry_date
                )
                console.success("API key persisted")
            self._token = None
            console.success(":white_check_mark: Authentication successful")

    def unauthenticate(self) -> None:
        self._api_key = None

    def execute(self, request: api_request.APIRequest) -> Dict[str, Any]:
        """Executes a request using the Authenticated GraphQL API.

        Args:
            request: The request to be executed

        Raises:
            ResponseError: When the API returns an error

        Returns:
            The API response
        """
        if self._token is not None:
            headers = {"Authorization": f"Token {self._token}"}
        elif self._api_key is not None:
            headers = {"X-Api-Key": f"{self._api_key}"}
        else:
            headers = None
            console.warning("No authentication credentials were provided.")

        response = self._sent_request(request, headers)
        if response.status_code != 200:
            logger.debug(
                "Response status code is %s: %s", response.status_code, response.content
            )
            raise runner.ResponseError(
                f'Response status code is {response.status_code}: {response.content.decode(errors="ignore")}'
            )
        data: Dict[str, Any] = response.json()
        errors = data.get("errors")
        if errors is not None and isinstance(errors, list):
            error = errors[0].get("message")
            raise runner.ResponseError(f"Response errors: {error}")
        else:
            return data

    def _sent_request(
        self, request: api_request.APIRequest, headers: Optional[Dict[str, str]] = None
    ) -> requests.Response:
        """Sends an API request."""
        if self._proxy is not None:
            proxy = {"https": self._proxy}
        else:
            proxy = None

        return requests.post(
            self.endpoint,
            data=request.data,
            files=request.files,
            headers=headers,
            proxies=proxy,
            verify=self._verify,
            timeout=runner.REQUEST_TIMEOUT,
        )

    def execute_ubjson_request(self, request: api_request.APIRequest) -> Dict[str, Any]:
        """Executes a request using the Authenticated GraphQL API.

        Args:
            request: The request to be executed

        Raises:
            ResponseError: When the API returns an error

        Returns:
            The API response
        """
        if self._token is not None:
            headers = {"Authorization": f"Token {self._token}"}
        elif self._api_key is not None:
            headers = {"X-Api-Key": f"{self._api_key}"}
        else:
            headers = None
            console.warning("No authentication credentials were provided.")

        if headers is not None:
            headers |= {"Content-type": "application/ubjson"}

        response = self._send_ubjson_request(request, headers)
        if response.status_code != 200:
            logger.debug(
                "Response status code is %s: %s", response.status_code, response.content
            )
            raise runner.ResponseError(
                f'Response status code is {response.status_code}: {response.content.decode(errors="ignore")}'
            )
        data: Dict[str, Any] = json.loads(response.content.decode())
        errors = data.get("errors")
        if errors is not None and isinstance(errors, list):
            error = errors[0].get("message")
            raise runner.ResponseError(f"Response errors: {error}")
        else:
            return data

    def _send_ubjson_request(
        self, request: api_request.APIRequest, headers: Optional[Dict[str, str]] = None
    ) -> requests.Response:
        """Sends an API request."""
        if self._proxy is not None:
            proxy = {"https": self._proxy}
        else:
            proxy = None

        return requests.post(
            self.endpoint,
            data=ubjson.dumpb(request.data),
            files=request.files,
            headers=headers,
            proxies=proxy,
            verify=self._verify,
            timeout=runner.REQUEST_TIMEOUT,
        )



"""IP address asset."""
import dataclasses
import ipaddress
from typing import Optional

from ostorlab.assets import asset


@dataclasses.dataclass
@asset.selector("v3.asset.ip")
class IP(asset.Asset):
    """IP Address target asset."""

    def __init__(
        self, host: str, version: Optional[int] = None, mask: Optional[str] = None
    ):
        self.host = host
        self.version = version or ipaddress.ip_address(self.host).version
        self.mask = mask

    def __str__(self) -> str:
        return f"{self.host}/{self.mask}"



"""Utils to handle scan risk rating."""
import enum
from typing import List


class RiskRating(enum.Enum):
    """Enumeration of the risk rating of a scan."""

    HIGH = "High"
    MEDIUM = "Medium"
    LOW = "Low"
    POTENTIALLY = "Potentially"
    HARDENING = "Hardening"
    SECURE = "Secure"
    IMPORTANT = "Important"
    INFO = "Info"

    @classmethod
    def has_value(cls, value: str) -> bool:
        return value in cls._member_map_

    @classmethod
    def values(cls) -> List[str]:
        return [key.lower() for key in cls._member_map_.keys()]


RATINGS_ORDER = {
    RiskRating.HIGH.name: 0,
    RiskRating.MEDIUM.name: 1,
    RiskRating.LOW.name: 2,
    RiskRating.POTENTIALLY.name: 3,
    RiskRating.HARDENING.name: 4,
    RiskRating.SECURE.name: 5,
    RiskRating.IMPORTANT.name: 6,
    RiskRating.INFO.name: 7,
}



"""Common definition types."""
import dataclasses
from typing import Optional, Union


@dataclasses.dataclass
class Arg:
    """Data class holding a definition.

    The value is always bytes to support all arg values. The type is defined by the type attribute.
    """

    name: str
    type: str
    value: Optional[Union[bytes, int, float, str, bool]] = None
    description: Optional[str] = None


@dataclasses.dataclass
class PortMapping:
    """Data class defining a port mapping source to destination"""

    source_port: int
    destination_port: int



"""Ostorlab main package."""
import logging
from rich.logging import RichHandler
from ostorlab.cli.rootcli import rootcli

FORMAT = "%(message)s"
logging.basicConfig(
    level="ERROR",
    format=FORMAT,
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=True)],
)

logger = logging.getLogger("CLI")


def main():
    rootcli(None)



"""This module triggers a scan using the ios_store asset."""
import logging
from typing import Tuple, Optional

import click

from ostorlab.cli.scan.run import run
from ostorlab.cli import console as cli_console
from ostorlab.assets import ios_store as ios_store_asset

console = cli_console.Console()
logger = logging.getLogger(__name__)


@run.run.command(name="ios-store")
@click.option("--bundle-id", multiple=True, required=False)
@click.pass_context
def ios_store(ctx: click.core.Context, bundle_id: Optional[Tuple[str]] = ()) -> None:
    """Run scan for a bundle_id."""
    runtime = ctx.obj["runtime"]
    assets = []
    if bundle_id != ():
        for bundle in bundle_id:
            assets.append(ios_store_asset.IOSStore(bundle_id=bundle))
    else:
        console.error("Command missing a bundle id.")
        raise click.exceptions.Exit(2)

    logger.debug("scanning assets %s", [str(asset) for asset in assets])
    runtime.scan(
        title=ctx.obj["title"],
        agent_group_definition=ctx.obj["agent_group_definition"],
        assets=assets,
    )
